<?php

/**
 * @file
 * Tokens provided by the Parameters module.
 */

use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\parameters\Entity\ParametersCollectionInterface;
use Drupal\parameters\Entity\ParametersCollectionStorage;
use Drupal\parameters\Parameter;
use Drupal\parameters\Plugin\Parameter\NullObject;

/**
 * Implements hook_token_info().
 */
function parameters_token_info(): array {
  $info = [];
  $info['types']['parameter'] = [
    'name' => t('Parameter'),
    'description' => t('Tokens provided by configured parameters.'),
  ];
  /** @var \Drupal\parameters\Entity\ParametersCollectionInterface $collection */
  foreach (ParametersCollectionStorage::get()->loadMultiple() as $collection) {
    $parameters = $collection->getParameters();
    if (!$parameters->count()) {
      continue;
    }
    $id = $collection->id();
    /** @var \Drupal\parameters\Plugin\ParameterInterface $parameter */
    foreach ($parameters as $name => $parameter) {
      $info['tokens']['parameter'][$id . ':' . $name] = [
        'name' => $parameter->getLabel() . ' (' . $parameter->getPluginDefinition()['label'] . ')',
        'description' => $parameter->getDescription(),
      ];
    }
  }
  return $info;
}

/**
 * Implements hook_tokens().
 */
function parameters_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata): array {
  $replacements = [];

  $parameter_tokens = $type === 'parameter' ? $tokens : \Drupal::token()->findWithPrefix($tokens, 'parameter');

  if ($parameter_tokens) {
    $storage = ParametersCollectionStorage::get();
    foreach ($parameter_tokens as $name => $original) {
      $parts = explode(':', $name, 2);
      if ($collection = $storage->load(reset($parts))) {
        array_shift($parts);
        if (empty($parts)) {
          continue;
        }
        $parameter = Parameter::get(reset($parts), $collection);
      }
      else {
        $context = isset($data[$type]) ? [$data[$type]] : (count($data) === 1 ? array_values($data) : []);
        $parameter = Parameter::get($name, ...$context);
      }

      if ($parameter instanceof NullObject) {
        // A null object represents a non-existent parameter.
        continue;
      }

      // The parameter plugin itself is responsible for safely handling input.
      // Therefore creating a Markup object here to avoid double escaping.
      $replacements[$original] = Markup::create($parameter->getProcessedData()->getString());
      $bubbleable_metadata->addCacheTags(\Drupal::entityTypeManager()->getDefinition(ParametersCollectionInterface::ENTITY_TYPE_ID)->getListCacheTags());
    }
  }

  return $replacements;
}
